require 'spec_helper'

describe Sentimeta::RestClient, :vcr do

  subject do
    module TestModule extend Sentimeta::RestClient; end
  end

  it { is_expected.to respond_to(:fetch) }

  describe '#generate_uri' do
    it 'works without options' do
      expect(
        subject.generate_uri(:spheres)
      ).to eq 'http://staging-api.sentimeta.com/api/v1/spheres'
    end

    it 'generate right uri for attributes' do
      expect(
        subject.generate_uri(:attributes, filter: 'actors', sphere: 'movies')
      ).to eq 'http://staging-api.sentimeta.com/api/v1/movies/attributes/actors'
    end

    it 'generate right uri for prices' do
      expect(
        subject.generate_uri(:prices, provider: 'booking', sphere: 'hotels')
      ).to eq 'http://staging-api.sentimeta.com/api/v1/hotels/prices/booking'
    end

    it 'puts id at the end of URI' do
      expect(
        subject.generate_uri(:prices, id: 12345, provider: 'booking', sphere: 'hotels')
      ).to eq 'http://staging-api.sentimeta.com/api/v1/hotels/prices/booking/12345'
    end
  end

  describe '#fetch' do
    it 'sends get request by default' do
      expect(::RestClient::Request).to receive(:execute)
        .with(hash_including(method: :get))
        .and_call_original

      subject.fetch 'spheres'
    end

    it 'sends right request' do
      expect(::RestClient::Request).to receive(:execute)
        .with(hash_including(url: 'http://staging-api.sentimeta.com/api/v1/hotels/catalog'))
        .and_call_original

      subject.fetch 'catalog', sphere: 'hotels'
    end

    it 'puts tokens to header' do
      headers = {
        'X-SERVER-ACCESS-TOKEN' => 'servertoken',
        'X-TOKEN' => 'usertoken'
      }
      Sentimeta.server_token = 'servertoken'
      expect(::RestClient::Request).to receive(:execute)
        .with(hash_including(headers: headers))
        .and_call_original

      subject.fetch 'catalog', sphere: 'hotels', token: 'usertoken'
    end

    it 'handle empty parameters normally' do
      expected_params = { param: :test }
      expect(::RestClient::Request).to receive(:execute)
        .with(hash_including(payload: expected_params))
        .and_call_original

      subject.fetch 'catalog', sphere: 'hotels', blabla: nil, param: :test
    end
  end

  it "responds to shortcut methods" do
    %i(get post put patch delete options).each do |method|
      expect(::RestClient::Request).to receive(:execute)
        .with(hash_including(method: method))
        .and_return(OpenStruct.new(code: 200, body: ''))
      subject.public_send(method)
    end
  end

end
